/*
    SETKISS.C

    Version 3.1 for Win32 15/1/1997 - Marco Savegnago IW3FQG

    This simple program turn on and off kissmode in many TNC directly from
    NT command prompt.

    Use it in a startup batch before running any program which need kiss mode
    tnc/operation.

    Usage SetKiss [command] <options>

    command:

    Bfilename specify BINARY filename to send

    Cx    specify com number 1,2,3 or 4.

    Ffilename specify ASCII filename to send

    In the file you can put any string or number or this command:

            -B          send BREAK
            -C          send CTRL-C
            -R          send CR
            -nn         send any ASCII char (es -9 send TAB)
            @filename   send filename (recoursing limited to 20 - 3 handle)

    Phhhh specify i/o port address in hex es: P03F8

    Sx    set COM speed (3)00 (1)200 (2)400 (4)800 (9)600
                              192(0)0 3(8)400 5(7)600 11(5)200 baud

    Tp    set TNC type  A = AEA PK88 PK232
                        O = Kantronics KPC's or KAM with old eprom
                        K = Kantronics KPC's or KAM with eprom >= 3.XX
                        T = TAPR TNC's and clone with std eprom
                        H = TAPR TNC's and clone with HOST eprom

    U|L  uninstall kiss using standard leave kiss sequence
                                (may not run in certain eprom)

    It was written for my personal use and for my personal needs!

    This is an HAMWARE software, no registration is required!

    ERROR_LEVEL:

           -1   - Show usage
            0   - OK, KISS (probably) set
            1   - Switch/Command error
            2   - TNC (probably) OFF
            3   - invalid com number
            4   - no filename specified
            5   - port address not valid
            6   - invalid TNC type
            7   - invalid speed

*/

//	CRTL include

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <conio.h>
#include <direct.h>

//	WIN32 includes
#include <windows.h>
#include <windef.h>
#include	<winbase.h>

#define OK_PORT         0x01
#define OK_SPEED        0x02
#define OK_TYPE         0x10
#define OK_FILE_ASCII   0x20
#define OK_FILE_BINARY  0x40


int			iTncType;
int			iComNumber;
HANDLE		hCom;
int			iSpeed;

char            szSendFilename[MAX_PATH];

const char  cszThisVersion[] = "3.1";

struct strBaudRate
{
	char 		cSpeed;
   int		iValue;
}

BaudRate[] =
{
    {  '3',  CBR_300     } ,
    {  '1',  CBR_1200    } ,
    {  '2',  CBR_2400    } ,
    {  '4',  CBR_4800    } ,
    {  '9',  CBR_9600    } ,
    {  '0',  CBR_19200   } ,
    {  '8',  CBR_38400   } ,
    {  '7',  CBR_57600   } ,
    {  '5',  CBR_115200  } ,
    {  '6',  CBR_256000  }
};

struct strKissCmd
{
    char type;
    char *string;
}

KissCmd[] =
{
{ 'N', "\xC0\xFF\xC0"     } ,  /* leave kiss string */
{ 'A', "CONMODE TRANS\r"
       "PP ON\r"
       "RAW ON\r"
       "HP OFF\r"
       "KISS ON\r"
       "HOST ON\r"        } ,  /* AEA */
{ 'T', "KISS ON\r"
       "RESTART\r"        } ,  /* TAPR TNC2 */
{ 'O', "KISSMODE ON\r"
       "RESET\r"          } ,  /* KANTRONICS 2.XX */
{ 'K', "INTFACE KISS\r"
       "RESET\r"          } ,  /* KANTRONICS 3.XX */

{ 'H', "\x11\x18\x1b@K\r" }    /* TF 2.1c (by Nord<>Link) for TNC2 */

};

int     htoi				(char *);
void    CloseComm       (void);
BOOL    OpenComm        (void);
int     SendKiss       	(int);
int     SendCom        	(char *);
void    Usage           (void);
void    CmdErrorExit  	(int);
void    ParseCmd       	(char *);
void    SendAsciiFile   (char *);
void    SendBinaryFile  (char *);
void    SendBreak       (void);

/*
** The main thing
*/

int main(int na,char **a)
{
  char*	p;
  int 	narg;
  int		i;
  char 	cmdline = 0;
  BOOL 	cmderror = FALSE;

  printf("\nSetKiss %s for Win32 - (c) 1990/1993/1996 Marco Savegnago IW3FQG  %s\n\n", cszThisVersion, __DATE__);

/*
**  See if command line is complete with argoment
*/

  for ( narg = 1; narg < na; narg++ )
  {
      p=a[narg];
      if ( *p == '-' || *p == '/')
      {
          p++;
          switch(toupper(*p))
          {
              case '?':
              case 'H':
                  Usage();
                  return(-1);

              case 'B': // custom kiss send by BINARY file
                  if (!*(p+1))
                      CmdErrorExit(2);

                  strcpy(szSendFilename,p);
                  cmdline |= OK_FILE_BINARY;
                  break;

              case 'C': // com
                  if (!*++p || *(p+1) || !isdigit(*p))
                  	CmdErrorExit(1);

                  iComNumber	=	atoi(p);
                  cmdline |= OK_PORT;
                  break;

              case 'F': // custom kiss send by file
                  if (!*(p+1))
                      CmdErrorExit(2);

                  strcpy(szSendFilename,p);
                  cmdline |= OK_FILE_ASCII;
                  break;

              case 'S': // speed
                  if (!*++p || *(p+1) || ! strpbrk(p, "3124908756"))
                                                      CmdErrorExit(4);

                  cmdline |= OK_SPEED;

                  for (i = 0; *p != BaudRate[i].cSpeed; i++)
                  ;
                  iSpeed=BaudRate[i].iValue;
                  break;

              case 'T': // type
                  if (!*++p || *(p+1))
                          CmdErrorExit(3);

                  *p	= (char) toupper(*p);

                  if (!strpbrk (p, "ATKOH"))
                              CmdErrorExit(5);

                  iTncType=toupper(*p);

                  cmdline |= OK_TYPE;
                  break;

              case 'L':
              case 'U': // unistall kiss
                  iTncType = 'N';
                  cmdline |= OK_TYPE;
                  break;

              default: // unknow switch
                  cmderror = TRUE;
                  printf("\aError! Invalid switch <%s>. Type /? for Help.\n",--p);
                  break;
          }
      }
      else
      {
          cmderror = TRUE;
          printf("\aError! Invalid command <%s>. Type /? for Help.\n",p);
      }
  }

  if (cmderror)
		return(1);

  if (! (cmdline & OK_PORT) || ! (cmdline & OK_SPEED) ||
                  ! (cmdline & OK_TYPE || cmdline & OK_FILE_ASCII || cmdline & OK_FILE_BINARY))
  {
          puts("\aError! Not enough operand! Type /? for help.\n");
          return(1);
  }

  if (FALSE == OpenComm())
  {
		CloseComm();
      return(2);
  }

  if (cmdline & OK_TYPE)
  		SendKiss(iTncType);

  if (cmdline & OK_FILE_ASCII)
        SendAsciiFile(szSendFilename);

  if (cmdline & OK_FILE_BINARY)
        SendBinaryFile(szSendFilename);

  CloseComm();

  return(TRUE);
}


/*
**  HexString to int without use assembly ...
*/

int htoi (char *p)
{
	int 	i;
   int 	ret=0;
   int	mult=1;
   int	ch;

   i=strlen(p);

   while(i--)
   {
   	ch = toupper(*(p+i));

      if(isalpha(ch))
      	ch -= ('A' - 10);
      else
      	ch -= '0';

      ret += ch * mult;
      mult *= 16;
	}

   return (ret);
}


/*
**  Close COM (reset DTR e RTS)
*/

void CloseComm (void)
{
	EscapeCommFunction( hCom, CLRDTR ) ;
   EscapeCommFunction( hCom, CLRRTS ) ;

	// purge any outstanding reads/writes and close device handle

	PurgeComm( hCom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ;

	CloseHandle( hCom ) ;
}


/*
** Open COM
*/

BOOL OpenComm(void)
{
   DCB				dcb;
   COMMTIMEOUTS	CommTimeOuts;
   char szCom[10];

   sprintf(szCom, "COM%i",iComNumber);

   if(( hCom = CreateFile
   (
      szCom,
      GENERIC_READ | GENERIC_WRITE,
      0,                    // exclusive access
      NULL,                 // no security attrs
      OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL,
      NULL
      )) == INVALID_HANDLE_VALUE)
    		return  FALSE;
   else
   {
      // setup device buffers

      SetupComm( hCom, 128, 128);

      // purge any information in the buffer

      PurgeComm( hCom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ;


      CommTimeOuts.ReadIntervalTimeout = MAXDWORD;
      CommTimeOuts.ReadTotalTimeoutMultiplier = 0 ;
      CommTimeOuts.ReadTotalTimeoutConstant = 0;
      CommTimeOuts.WriteTotalTimeoutMultiplier = 0 ;
      CommTimeOuts.WriteTotalTimeoutConstant = 0;

      SetCommTimeouts( hCom, &CommTimeOuts ) ;

      dcb.DCBlength = sizeof( DCB ) ;

      GetCommState( hCom, &dcb ) ;

      dcb.BaudRate    = iSpeed;
      dcb.ByteSize    = 8;
      dcb.Parity      = NOPARITY;
      dcb.StopBits    = ONESTOPBIT;

      dcb.fDtrControl = DTR_CONTROL_ENABLE ;
      dcb.fRtsControl = RTS_CONTROL_ENABLE ;

// 	other various settings

      dcb.fBinary = TRUE ;
      dcb.fParity = TRUE ;

      if(SetCommState( hCom, &dcb ))
      {
        EscapeCommFunction( hCom, SETDTR );
        EscapeCommFunction( hCom, SETRTS );
      }
	}

    return TRUE;
}


/*
**  Select and then send Kiss string
*/

int SendKiss(int	iTncType)
{
    register int i;

    for (i = 0;iTncType!=KissCmd[i].type;i++)
    ;

    SendCom(KissCmd[i].string);

    printf("Kissmode (probably) %sset!\n",(iTncType!='N') ? "" : "un");

    return(TRUE);
}


/*
**  Send kiss string to comport
*/

int SendCom(char *s)
{
   DWORD	dwToWrite	=	strlen(s);

    return  WriteFile(hCom, s, dwToWrite, &dwToWrite, NULL);
}


/*
**  Print Usage
*/

void Usage (void)
{

const	char usage_str[] =
     "Set/Reset Kiss mode utility for use with the most TNC's type\n\n"

     "This is am HAMWARE program, so it can be copied/modify freely but not for\n"
     "commercial purposes!\n\n"

     "Usage SetKiss [/|-command]\n\n"

     "command:\n"
     "/Bfilename specify BINARY filename to send.\n"
     "/Cx        specify com number 1,2,3 ...\n"
     "/Ffilename specify ASCII filename to send.\n"
     "/Sx        set COM speed (3)00 (1)200 (2)400 (4)800 (9)600\n"
     "\t\t\t\t192(0)0 3(8)400 5(7)600 11(5)200 25(6)000 baud\n\n"

     "/Tp        set TNC type  A = AEA PK88 PK232\n"
     "\t\t\tO = Kantronics KPC's or KAM with old eprom\n"
     "\t\t\tK = Kantronics KPC's or KAM with eprom >= 3.XX\n"
     "\t\t\tT = TAPR TNC's and clone with std eprom\n"
     "\t\t\tH = TAPR TNC's and clone with HOST eprom\n\n"
     "/U|L       uninstall kiss using standard leave kiss sequence"

     ;

     puts(usage_str);
}


/*
**  Print command error then exit with error level
*/

void CmdErrorExit (int num)
{
    char *str[] =
    {
        "invalid com number",
        "no filename specified",
        "port address specification is not valid",
        "invalid speed",
        "invalid TNC type"
    };


    printf ("\nError %s \a\n",str[num-1]);
    exit(num+2);
}


/*
**  Parse command in file
*/

void ParseCmd (char *str)
{
    char cmd[10];

    if (*str=='-')
    {
        switch(toupper(*++str))
        {
            case 'B':
                SendBreak();
                break;
            case 'C':
                SendCom("\x03");
                break;
            case 'R':
                SendCom("\r");
                break;
            default:
                if(isdigit(*str))
                {
                     sprintf(cmd,"%c",atoi(str));
                     SendCom(cmd);
                }
                break;
        }
    }
    else if (*str == '@' && strlen(str) > 1)  // Max file handle is 20
                                    SendAsciiFile(str);
    else
    {
    	SendCom(str);
        SendCom(" ");
    }
}

/*
**  Send file to COM
*/

void SendAsciiFile (char *filename)
{
    FILE *f;
    char cmd[256];

    filename++;

    if((f=fopen(filename,"rt"))==NULL)
    {
        printf("\n\aError opening file %s\n",filename);
        return;
    }

    while(!feof(f))
    {
        if(EOF==fscanf(f,"%255s",cmd))
        	break;

        ParseCmd(cmd);
    }

    fclose(f);
}

/*
**  Send file to COM
*/

void SendBinaryFile (char *filename)
{
    FILE *f;
    int ch;

    filename++;

    if((f=fopen(filename,"rb"))==NULL)
    {
        printf("\n\aError opening file %s\n",filename);
        return;
    }

    while(!feof(f))
    {
        if(EOF==fgetc(f))
        	break;

        WriteFile(hCom, &ch, 1, NULL, NULL);
    }

    fclose(f);
}

/*
**  Send BREAK
*/

void SendBreak(void)
{
    EscapeCommFunction( hCom, SETBREAK);
    Sleep(300);
    EscapeCommFunction( hCom, CLRBREAK);
}

